@@ -94,8 +94,8 @@ class TabSeparatedWithNamesAndTypesFormatter(object):
         elif existing_type.startswith(arr) and new_type.startswith(arr):
             return 'Array(%s)' % self.generalize_type(existing_type[len(arr):-1], new_type[len(arr):-1])
         elif existing_type.startswith(ma) and new_type.startswith(ma):
-            existing_spec = existing_type[len(ma),-1].split(',')
-            new_spec = new_type[len(ma), -1].split(',')
+            existing_spec = existing_type[len(ma):-1].split(',')
+            new_spec = new_type[len(ma): -1].split(',')
             return 'Map(%s,%s)' % (self.generalize_type(existing_spec[0].strip(), new_spec[0].strip()), self.generalize_type(
                 existing_spec[1].strip(), new_spec[1].strip()))
         elif existing_type.startswith(arr) or new_type.startswith(arr):
@@ -143,8 +143,8 @@ class TabSeparatedWithNamesAndTypesFormatter(object):
         elif existing_type.startswith(arr) and new_type.startswith(arr):
             return self.is_compatible_type(existing_type[len(arr):-1], new_type[len(arr):-1])
         elif existing_type.startswith(ma) and new_type.startswith(ma):
-            existing_spec = existing_type[len(ma),-1].split(',')
-            new_spec = new_type[len(ma), -1].split(',')
+            existing_spec = existing_type[len(ma):-1].split(',')
+            new_spec = new_type[len(ma): -1].split(',')
             return self.is_compatible_type(existing_spec[0].strip(), new_spec[0].strip()) \
                    and self.is_compatible_type(existing_spec[1].strip(),
                                               new_spec[1].strip())
@@ -340,6 +340,8 @@ class TabSeparatedWithNamesAndTypesFormatter(object):
                     return '[]'
                 return '[%s]' % ','.join([self.formatfield(x, type[6:-1], name, True) for x in value])
             if type.startswith('Map(') and type.endswith(')'):
+                if value is None:
+                    return '{}'
                 spec=type[4:-1].split(',')
                 if len(spec) != 2:
                     raise Exception('Cannot format type %s, it is either malformed or too nested for this simple '
@@ -388,45 +390,27 @@ class TabSeparatedWithNamesAndTypesFormatter(object):
                 return None
             tmp = dt.datetime.strptime(value, '%Y-%m-%d %H:%M:%S')
             if type.startswith('DateTime('):
-                tmp = tmp.astimezone(pytz.timezone(type.split('(')[1][2:-3]))
+                tmp = tmp.replace(tzinfo=pytz.timezone(type.split('(')[1][2:-3]))
             return tmp
-        if 'Array' in type:
-            if value == '[]':
-                return []
+        if type.startswith('Array('):
+            value = value.replace(',,', ",'',") # workaround empty string clickhouse bug
+            parts = ast.literal_eval(value)
 
-            # Handle arrays with strings containing commas, like ['abc','d,ef']
-            parts = []
-            acc = None
-            for part in value[1:-1].split(','):
-                stripped = part.rstrip().lstrip()
-
-                if len(stripped) == 0:
-                    parts.append(part)
-                    continue
-
-                if stripped[0] == "'" and stripped[-1] != "'":
-                    acc = part
-                    continue
-
-                if acc is not None:
-                    if stripped[-1] == "'":
-                        parts.append(acc + ',' + part)
-                        acc = None
+            def handle_dates(val, typ):
+                if typ == 'Date' or typ == 'DateTime':
+                    return [self.unformatfield(x, typ) for x in val]
+                else:
+                    if typ.startswith('Array('):
+                        return [handle_dates(x, typ[6:-1].strip()) for x in val]
                     else:
-                        acc += ',' + part
-                    continue
+                        return val
 
-                parts.append(part)
-
-            if acc is not None:
-                raise Exception('Cannot deserialize %s' % value)
-
-            return [self.unformatfield(x, type[6:-1]) for x in [y[1:-1] if len(y) >= 2 and y[0]=="'" and y[-1]=="'" else y for y in parts]]
+            return handle_dates(parts, type[6:-1].strip())
 
         if type.startswith('Map(') and type.endswith(')'):
             spec=type[4:-1].split(',')
             if len(spec) != 2:
-                raise Exception('Cannot format type %s, it is either malformed or too nested for this simple '
+                raise Exception('Cannot unformat type %s, it is either malformed or too nested for this simple '
                                 'driver' % (type))
             x = ast.literal_eval(value)
             spec[0] = spec[0].strip()
@@ -468,7 +452,7 @@ class TabSeparatedWithNamesAndTypesFormatter(object):
                         value = value[:-1]
                     if value == '0000-00-00 00:00:00' or value == '1970-01-01 86:28:16':
                         return None
-                    return dt.datetime.strptime(value, '%Y-%m-%d %H:%M:%S').astimezone(tz)
+                    return dt.datetime.strptime(value, '%Y-%m-%d %H:%M:%S').replace(tzinfo=tz)
             else:
                 def to_datetime(value):
                     if value.startswith("'"):
